package minademo;

import java.net.*;
import java.nio.charset.*;
import org.apache.mina.core.service.*;
import org.apache.mina.core.session.*;
import org.apache.mina.core.future.*;
import org.apache.mina.core.filterchain.*;
import org.apache.mina.transport.socket.nio.*;
import org.apache.mina.filter.codec.*;
import org.apache.mina.filter.codec.textline.*;
import org.apache.mina.filter.logging.*;
import org.slf4j.*;

public class MinaTimeClient {
	private static final int PORT = 8080;
	public static final long TIMEOUT = 5000;
	private static Logger logger = LoggerFactory.getLogger(MinaTimeClient.class);
	
	public static void configClient(IoConnector connector) {
		IoSessionConfig config = connector.getSessionConfig();
		config.setUseReadOperation(true);
		config.setReadBufferSize(2048);
	}
	
	public static NioSocketConnector getConnector() {
		NioSocketConnector connector = new NioSocketConnector();
		connector.getFilterChain().addLast("logger", new LoggingFilter());
		TextLineCodecFactory codecFactory = new TextLineCodecFactory(Charset.forName("UTF-8"));
		IoFilter codec = new ProtocolCodecFilter(codecFactory);
		connector.getFilterChain().addLast("codec", codec);
		connector.setConnectTimeoutMillis(TIMEOUT);
		connector.setHandler(new MinaTimeClientHandler());
		configClient(connector);
		return connector;
	}
	
	public static String read(IoSession session) {
		ReadFuture readFuture = session.read();
		readFuture.awaitUninterruptibly();
		String msg = readFuture.getMessage().toString();
		return msg;
	}
	
	public static void main(String[] args) {
		NioSocketConnector connector = getConnector();
		ConnectFuture cf = connector.connect(new InetSocketAddress("127.0.0.1", PORT));
		cf.awaitUninterruptibly();
		if (!cf.isConnected()) {
			connector.dispose();
			return;
		}
		logger.info("Client is ready...");
		
		IoSession session = cf.getSession();
		session.write("Hi Server!");
		session.write("quit");
		String content = read(session);
		logger.info("receive message: {}", content);
		session.getCloseFuture().awaitUninterruptibly();
		connector.dispose();
	}
}

